/*
 * Copyright (C) 2013 Pablo Taboada <pablo[dot]taboada[at]gmail[dot]com>
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package uoc.edu.ptaboada.tthome;

import java.util.ArrayList;
import java.util.Locale;
import java.util.Random;

import org.apache.http.client.methods.HttpGet;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import uoc.edu.ptaboada.tthome.TareaHttp.GestorTareaHttp;
import uoc.edu.ptaboada.tthome.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;

import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Activity principal de la aplicación.
 * Consiste en una ListActivity con una ActionBar desde la que se accede a las distintas opciones:
 * 	- Centros: ver el listado de centros
 *  - Empleados: ver el listado de empleados
 *  - Turnos: ver el listado de turnos
 *  - Configuración: establecer los datos de acceso al servidor (usuario y clave)
 *  
 *  Al ser creada verifica contra el servidor la validez de los datos de autenticación
 * 
 * @author Pablo Taboada
 */
public class HomeActivity extends ListActivity {
	private static final String URL_BASE = "http://android.tamtamhome.com/";
	private SharedPreferences pref;
	private TTApplication tt;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		ActionBar actionBar = getActionBar();
		actionBar.show();
		
		// Validación de datos de acceso (usuario+clave o token)
		valida();
	}
	
	// Evita que se vuelva a producir la validación al rotar el dispositivo:
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.mainmenu, menu);
		return true;
	}

	/**
	 * Validación de usuario y clave o token para conexión con el servidor.
	 * 
	 * Usuario y clave se almacenan en las SharedPreferences de la aplicación, 
	 * de modo que son persistentes aunque se cierre la misma.
	 * 
	 * El token tiene una caducidad de 1 día y se genera al arrancar la aplicación con la primera
	 * autenticación válida en el servidor. A partir de ese momento, se envía el token con cada petición.
	 * Si el servidor responde indicando que el token es erróneo, se genera uno nuevo.
	 */
	private void valida(){
		pref = PreferenceManager.getDefaultSharedPreferences(this);
		tt = (TTApplication)getApplication();
		
		String u = pref.getString("user", "");
		String c = pref.getString("clave", "");
		String t = tt.token;
		String param = "";
		
		if( u.isEmpty() || c.isEmpty() ){
			alertaAutenticacion(getString(R.string.no_usuario));
			return;
		}else{
			if(t.isEmpty()){
				param = "u="+u+"&c="+c;
			}else{
				param = "t="+t;
			}
		}
		
		// La petición HTTP se realiza en segundo plano para no bloquear el hilo principal de la aplicación
		TareaHttp task = new TareaHttp();
        task.setGestorTarea(new GestorTareaHttp() {
			public void tareaExito(JSONArray json) {
				try {
					JSONObject jo = new JSONObject(json.getString(0));
					String res = jo.getString("res");
					String val = jo.getString("val");

					if(res.equals("OK")){	// validación OK: guardamos el token recibido
						((TTApplication)getApplication()).token = val;
						makeToast(getString(R.string.usuario_ok));
					}else{					// validación incorrecta:
						if(val.equals("USER_ERR")){
							alertaAutenticacion(getString(R.string.usuario_err));
						}
						if(val.equals("TOKEN_ERR")){
							// reseteamos el token local y volvemos a solicitar uno nuevo
							((TTApplication)getApplication()).token = "";
							valida();
						}
					}
				} catch (JSONException e) {
					Log.e(getClass().getSimpleName(),e.toString());
					// Cualquier respuesta no esperada lanzará esta excepción 
					// al no recibirse datos con formato JSON
					makeToast(getString(R.string.error_conexion));
					return;
				}
			}
			public void tareaFracaso() {
	    	   	makeToast(getString(R.string.error_conexion));
	        }
        });
        task.execute(new HttpGet(URL_BASE+"valida.php?"+ param
        		+ "&sid="+(new Random()).nextInt()));
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item){
		ActionBar ab = this.getActionBar();
		ab.setTitle(item.getTitle());
		if (item.getItemId() == R.id.item_turnos) {
			ab.setIcon(R.drawable.img_turnos);
			cargaLista(tt.listaTurnos(),Turno.class);
		} else if (item.getItemId() == R.id.item_centros) {
			ab.setIcon(R.drawable.img_centros);
			cargaLista(tt.listaCentros(),Centro.class);
		} else if (item.getItemId() == R.id.item_empleados) {
			ab.setIcon(R.drawable.img_empleados);
			cargaLista(tt.listaEmpleados(),Empleado.class);
		} else if (item.getItemId() == R.id.item_settings) {
			Intent settingsActivity = new Intent(getBaseContext(),Preferences.class);
			startActivity(settingsActivity);
		}
		return true;
	}
	
	/**
	 * CARGA el ListView con los valores ya existentes en la TTApplication
	 * 
	 * @param al	ArrayList con los datos a cargar
	 * @param clase	Clase de datos que se cargarán (Centro, Empleado o Turno)
	 */
	private void cargaLista(ArrayList<String> al, final Class<?> clase){
		// Si no tenemos datos, los buscamos en el servidor:
		if(al.isEmpty()){
			getLista(clase);
		}
		
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(HomeActivity.this, 
				android.R.layout.simple_list_item_1, al);
		ListView list = (ListView) getListView();
		list.setAdapter(adapter);
        list.setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view,
                    int position, long id) {
                String item = ((TextView)view).getText().toString();
                Intent in = new Intent(getApplicationContext(), clase);
				in.putExtra("nome", item);
                startActivity(in);
            }
        });
	}
	
	/**
	 * Pide datos al servidor para cargar las listas de TTApplication
	 * Los datos recibidos se almacenan en las variables de Aplicación, de modo que estén
	 * disponibles para las siguientes veces. 
	 * 
	 * Al finalizar, llama a cargaLista para mostrar los datos.
	 * 
	 * @param clase	Clase de datos a cargar en la lista (Centro, Empleado o Turno)
	 */
	private void getLista(final Class<?> clase){
		final String que = clase.getSimpleName();
		// Al ser una petición HTTP, lo hacemos en segundo plano
		TareaHttp task = new TareaHttp();
        task.setGestorTarea(new GestorTareaHttp() {
			public void tareaExito(JSONArray json) {
				try {
					// VERIFICA EL RESULTADO por si llegan errores:
					JSONObject j = new JSONObject(json.getString(0));
					if(j.has("res")){
						if(j.getString("val").equals("0")){
							// La consulta SQL no devuelve ningún dato:
							makeToast(getString(R.string.no_datos));
							return;
						}else{ 
							// TOKEN incorrecto: lo resetea y solicita uno nuevo:
							((TTApplication)getApplication()).token = "";
							valida();
							return;
						}
					}
					
					// Si llega hasta aquí, los datos de json deben ser válidos para cargar la lista
					for (int i = 0; i < json.length(); i++) {
						JSONObject jo = new JSONObject(json.getString(i));				

					    // guardamos cada elemento en la variable global de TTApplication			    
					    if(que.equals("Turno")){
					    	Turno tur = new Turno(jo.getInt("id"),
					    			jo.getString("ofi"),
					    			jo.getString("emp"),
					    			jo.getString("ini"),
					    			jo.getString("fin"),
					    			jo.getInt("ok"));
					    	tt.turnos.add(tur);
					    	continue;
					    }
					    if(que.equals("Centro")){
					    	Centro cen = new Centro(jo.getInt("id"),
					    			jo.getString("nom"),
					    			jo.getString("dir"),
					    			jo.getString("pob"),
					    			jo.getString("pro"),
					    			jo.getString("cp"));
					    	tt.centros.add(cen);
					    	continue;
					    }
					    if(que.equals("Empleado")){
					    	Empleado emp = new Empleado(jo.getInt("id"),
					    			jo.getString("nom"),
					    			jo.getString("tel"),
					    			jo.getString("cod"));
					    	tt.empleados.add(emp);
					    	continue;
					    }
					}
				} catch (JSONException e) {
					Log.e(getClass().getSimpleName(),e.toString());
					makeToast(getString(R.string.error_json));
					return;
				}

				// mostramos la lista 
				if(que.equals("Turno")){
					cargaLista(tt.listaTurnos(), clase);
				}
				if(que.equals("Centro")){
					cargaLista(tt.listaCentros(), clase);
				}
				if(que.equals("Empleado")){
					cargaLista(tt.listaEmpleados(), clase);
				}
			}
			public void tareaFracaso() {
	    	   	makeToast(getString(R.string.error_conexion));
	        }
        });
        task.execute(new HttpGet(URL_BASE+"?t="+ tt.token 
        		+ "&tbl="+ que.substring(0,1).toLowerCase(new Locale("es_ES"))
        		+ "&sid="+(new Random()).nextInt()) );
	}
	
	/**
	 * Muestra un mensaje en pantalla durante 3 segundos
	 * 
	 * @param mensaje	mensaje a mostrar
	 */
	public void makeToast(String mensaje) {
		Toast.makeText(this, mensaje, Toast.LENGTH_SHORT).show();
	}
	
	/**
	 * Muestra en pantalla un cuadro de diálogo con un mensaje y dos botones (OK y CANCEL)
	 * Está siempre vinculado al erro de autenticación, permitiendo ir directamente
	 * a las opciones de configuración si se pulsa el OK.
	 * 
	 * @param mensaje mensaje a mostrar
	 */
	private void alertaAutenticacion(String mensaje){
		AlertDialog.Builder alert = new AlertDialog.Builder(HomeActivity.this);
		alert.setMessage(mensaje+"\n\n"+getString(R.string.deseas))
			.setCancelable(false)
			.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					dialog.cancel();
					Intent in = new Intent(getBaseContext(),Preferences.class);
					startActivity(in);
				}
			  })
			.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					dialog.cancel();
				}
			});
 		alert.create().show();
	}
}